function [ RTT , signal_var] = round_trip_time( v )
%round_trip_time Returns time it takes for the us signal to travel back and forth in number of samples.
%   This function switches between detecting RTT by comparing echo1 and
%   echo2 and comparing echo1 with a stored reference echo1. This makes it
%   more robust to noise. If the signal strength is good both echoes are
%   used. When the signal strength is weak the reference echo1 is used. The
%   Echo1 reference is updated when both echoes are used to detect RTT and
%   when sudden jumps occure in the detected RTT, as this indicates that
%   the signal has changed and a new reference is needed. 



    number_of_measurements=size(v,1);
    
    RTT=zeros(1,size(v,1));
    %shift=zeros(1,size(v,1));
    
    signal_var=zeros(size(v,1),1);

    remove_start_noise_index=700; %Remove all values in front of this index.
    v(:,1:remove_start_noise_index)=zeros(size(v,1),remove_start_noise_index);
    period_length=14; %% Number of samples in a period. 14 because signalfrequency 1MHz and sampling frequency 14 MHz.

    echo1_padding=10*period_length;
    echo2_padding=10*period_length;
    echo1=zeros(size(v,1), 5*echo1_padding+1);
    echo2=zeros(size(v,1), 5*echo2_padding+1);

    %%%%Loops trough all the measurements and detects the roundtrip time.
    kalman_dim=1;
    kalman = struct('RTT', zeros(2*kalman_dim,1), 'F', [eye(kalman_dim), eye(kalman_dim);zeros(kalman_dim), eye(kalman_dim)], 'P', 1000*eye(2*kalman_dim), 'Q', eye(2*kalman_dim), 'R', eye(kalman_dim), 'y', zeros(kalman_dim,1), 'H', [eye(kalman_dim), zeros(kalman_dim)], 'S', zeros(kalman_dim), 'K', zeros(2*kalman_dim,kalman_dim), 'I', eye(2*kalman_dim));
    
    signal_var_threshold=0;
    RTT_offset=0;
    v_ref=v(1,:);
    RTT_ref=0;
    for i=1:number_of_measurements,
  
        %%%%%Size of the window used to look for echo1
        size_window1=800;
        %%%%%Index where the echo1 window begins
        t1=700;

        %%%%%Create a vector containing the energy in the signal in a given
        %%%%%window
        wave_energy_echo1=signal_energy_window(v(i,t1:t1+size_window1-1), 2*period_length);

        %Find the index of the maximum in the echo1 window. This should be
        %somwhere within echo1.
        [echo_strength_echo1, echo_index_echo1]=max(wave_energy_echo1);
        %Find the start of echo1 based on a dynamic threshold value
        [ start_echo1, ~ ] = echoStartAndEndIndex(wave_energy_echo1, echo_index_echo1);
 
        %%%%%Size of the window used to look for echo2
        size_window2=size_window1;
        %%%%%Index where the echo2 window begins. This depends on where the
        %%%%%start of echo2 was found.
        t2=2*(t1+start_echo1)-round(size_window2/2);
        %%%%%Create a vector containing the energy in the signal in a given
        %%%%%window
        wave_energy_echo2=signal_energy_window(v(i,t2:t2+size_window2-1), 2*period_length);

        %%%%%Find max value in the echo 2 window. Here a gaussian window is
        %%%%%used to favorise the area we expect the echo to occure based on
        %%%%%the previous detection of echo1.
        [~, echo_index_echo2]=max(wave_energy_echo2.*gausswin(size_window2)');

        %Find the start of echo2 based on a dynamic threshold value
        [ start_echo2, ~ ] = echoStartAndEndIndex(wave_energy_echo2, echo_index_echo2);

        %%%%%Add som area around the start of each echo.
        end_echo1=t1+start_echo1+4*echo1_padding;
        end_echo2=t2+start_echo2+4*echo2_padding;
        
        start_echo1=t1+start_echo1-echo1_padding;
        start_echo2=t2+start_echo2-echo2_padding;
        if end_echo2>length(v(i,:)),
            end_echo2=length(v(i,:));
            start_echo2=end_echo2 -5*echo2_padding;
        end 

        %%%%%Store the window used to compare echo1 and echo2 in two vectors.
        echo1(i, :)=v(i,start_echo1:end_echo1);
        echo2(i, :)=v(i,start_echo2:end_echo2);
        
        signal_var(i)=var(v(i,remove_start_noise_index+1:end));
        
                
%         if i>1,
%             [~,shift(i)]=minerr(v(i, :),v(i-1, :)); 
%             shift(i)=improve_shift_est( v(i, :),v(i-1, :), -shift(i) );
%         else
%             shift(i)=0;
%         end

        
        signal_var_threshold=max(signal_var_threshold, 0.99*signal_var(i));
%         if i>573 && i < 577,
%            signal_var(i) 
%            signal_var_threshold
%            abs(RTT_diff)
%         end
        if signal_var(i)>signal_var_threshold,
            %%%%%Detect the index shift corresponding to the best fit between echo1
            %%%%%and echo2.
            [~,index]=minerr(normalize(echo1(i, :)),normalize(echo2(i, :))); %normalizes the rest of the signal excluding echo 1 and compare echo 1 to it.

            %%%%%Use the index found with the first estimate of start echo1 and start echo2 to find the delay time. 
            RTT(i)=index+start_echo2-start_echo1;
            %RTT(i) = improve_RTT_est( v(i,:), RTT(i) );

         
            
            
            v_ref=v(i,:);
            RTT_ref=RTT(i);

            
        else
            %%%%%Detect the index shift corresponding to the best fit between echo1
            %%%%%and echo2.
            [~,index]=minerr(v_ref,v(i, :)); %normalizes the rest of the signal excluding echo 1 and compare echo 1 to it.

            %%%%%Use the index found with the first estimate of start echo1 and start echo2 to find the delay time. 
            RTT(i)=index+RTT_ref;
            
            if i>1,
                RTT_diff=RTT(i)-RTT(i-1);
            else
                RTT_diff=0;
            end   
            if abs(RTT_diff)>=12 && abs(RTT_diff)<=16,
                disp(i)
    %           RTT_offset=RTT_offset+RTT_diff;%+shift(i);
                v_ref=v(i-1,:);
                RTT_ref=RTT(i-1);
                
                [~,index]=minerr(v_ref,v(i, :)); %normalizes the rest of the signal excluding echo 1 and compare echo 1 to it.
                RTT(i)=index+RTT_ref;
               
            end
            
           
        end
        

    end




end